local _G = GLOBAL
local _T = _G.TUNING

---- Check whether RoG is enabled
local IsRoG = _G.IsDLCEnabled(_G.REIGN_OF_GIANTS)
local IsSW = _G.IsDLCEnabled(_G.CAPY_DLC)
local IsHamlet = _G.IsDLCEnabled(_G.PORKLAND_DLC)

local dapperGainEffect = GetModConfigData("DapperGainEffect")
local dapperLossEffect = GetModConfigData("DapperLossEffect")
local auraGainEffect = GetModConfigData("AuraGainEffect")
local auraLossEffect = GetModConfigData("AuraLossEffect")
local rainEffect = GetModConfigData("RainEffect")
local drivableEffect = GetModConfigData("DrivableEffect")
local mountGainEffect = GetModConfigData("MountGainEffect")
local mountLossEffect = GetModConfigData("MountLossEffect")
local poisonEffect = GetModConfigData("PoisonEffect")
local oldSANITY_GHOST_PLAYER_DRAIN = 0
local flowerSanityChange = GetModConfigData("FlowerSanityChange")
local seashellSanityChange = GetModConfigData("SeashellSanityChange")
local evilFlowerSanityChange = GetModConfigData("EvilFlowerSanityChange")

-- Calculate night_time, for use in calculations for day-cycle sanity gain/loss
local night_time = _T.NIGHT_SEGS_DEFAULT * _T.SEG_TIME
local day_time = _T.DAY_SEGS_DEFAULT * _T.SEG_TIME
_T.SANITY_DAY_GAIN = GetModConfigData("DayTick")
_T.SANITY_NIGHT_LIGHT = (GetModConfigData("NightLight")/(night_time*20))
_T.SANITY_NIGHT_MID = (GetModConfigData("NightMid")/(night_time*20))
_T.SANITY_NIGHT_DARK = (GetModConfigData("NightDark")/(night_time*2))
_T.SANITY_PLAYERHOUSE_GAIN = (GetModConfigData("PlayerHouseGain")/(day_time*32))
_T.SANITY_EFFECT_RANGE = GetModConfigData("AuraEffectRange")

-- Get required components
local require = _G.require
local easing = require("easing")

local function RecalcHam(self, dt)
	local total_dapperness = self.dapperness or 0
	local mitigates_rain = false
	
	local empty_slots = 3
	local tempDappernessVal = 0
	for k,v in pairs (self.inst.components.inventory.equipslots) do
		if v.components.equippable then
			empty_slots = empty_slots - 1
			-- This has been modified, to apply gain/loss per item evaluated, instead of doing it on the total
			tempDappernessVal = v.components.equippable:GetDapperness(self.inst)
			if tempDappernessVal > 0.0 then
				total_dapperness = total_dapperness + ((tempDappernessVal / 100) * dapperGainEffect)
			elseif tempDappernessVal < 0.0 then
				total_dapperness = total_dapperness + ((tempDappernessVal / 100) * dapperLossEffect)
			end
		end		
	end
	
	total_dapperness = total_dapperness * self.dapperness_mult 

	local moisture_delta = self:GetMoistureDelta()

	local dapper_delta = total_dapperness*_T.SANITY_DAPPERNESS
	
	local light_delta = 0
	local lightval = self.inst.LightWatcher:GetLightValue()
	
	local day = _G.GetClock():IsDay() and not _G.GetWorld():IsCave()
	local isInside = false
	local isInPlayerHouse = false

    if self.inst == _G.GetPlayer() then
    	-- In House - gain sanity 
		-- in other interiors no day/night effect
		-- outside, general day/night drain
		isInside = _G.TheCamera.interior
		isInPlayerHouse = _G.GetInteriorSpawner() and _G.GetInteriorSpawner():InPlayerRoom() or false
	end

	if isInPlayerHouse then
		light_delta = _T.SANITY_PLAYERHOUSE_GAIN
	elseif not isInside then
		if day then 
			light_delta = _T.SANITY_DAY_GAIN
		else	
			local highval = _T.SANITY_HIGH_LIGHT
			local lowval = _T.SANITY_LOW_LIGHT

			if lightval > highval then
				light_delta =  _T.SANITY_NIGHT_LIGHT
			elseif lightval < lowval then
				light_delta = _T.SANITY_NIGHT_DARK
			else
				light_delta = _T.SANITY_NIGHT_MID
			end

			light_delta = light_delta*self.night_drain_mult
		end
	end
	
	local aura_delta = 0
	local x,y,z = self.inst.Transform:GetWorldPosition()
	local ents = _G.TheSim:FindEntities(x,y,z, _T.SANITY_EFFECT_RANGE, nil, {"FX", "NOCLICK", "DECOR","INLIMBO"} )
	for k,v in pairs(ents) do 
		if v.components.sanityaura and v ~= self.inst then
			local distsq = self.inst:GetDistanceSqToInst(v)
			local aura_val = v.components.sanityaura:GetAura(self.inst)/math.max(1, distsq)
			if aura_val < 0 then
				aura_val = aura_val * self.neg_aura_mult
			end
			
			-- This is customized, to apply gain/loss per aura evaluated, instead of doing it on the total
			if aura_val > 0.0 then
				aura_val = ((aura_val / 100) * auraGainEffect)
			elseif aura_val < 0.0 then
				aura_val = ((aura_val / 100) * auraLossEffect)
			end

			aura_delta = aura_delta + aura_val
		end
	end
	
 	local drivabledelta = 0 
 	if self.inst.components.driver then 
 		local vehicle = self.inst.components.driver.vehicle
 		if vehicle then 
 			if vehicle.components.drivable then
 				drivabledelta = vehicle.components.drivable:GetSanityDrain()
			elseif vehicle.components.searchable then
				drivabledelta = vehicle.components.searchable:GetSanityDrain()
			end
 		end 
 	end 

 	local poisondelta = 0
 	if self.inst.components.poisonable and self.inst.components.poisonable:IsPoisoned() then
 		poisondelta = -self.inst.components.poisonable.damage_per_interval * _T.POISON_SANITY_SCALE
 	end
	
	local mount = self.inst.components.rider and self.inst.components.rider:IsRiding() and self.inst.components.rider:GetMount() or nil
    if mount and mount.components.sanityaura  then
        local aura_val = mount.components.sanityaura:GetAura(self.inst)
		
		-- This is customized, to apply an effect depending on whether the aura is positive or negative.
		if aura_val > 0.0 then
			aura_val = ((aura_val / 100) * mountGainEffect)
		elseif aura_val < 0.0 then
			aura_val = ((aura_val / 100) * mountLossEffect)
		end
		
        aura_delta = aura_delta + aura_val
    end
	
	drivabledelta = ((drivabledelta / 100) * drivableEffect)
	poisondelta = ((poisondelta / 100) * poisonEffect)
	moisture_delta = ((moisture_delta / 100) * rainEffect)
 
	self.rate = (dapper_delta + moisture_delta + light_delta + aura_delta + drivabledelta + poisondelta)
	
	if self.custom_rate_fn then
		self.rate = self.rate + self.custom_rate_fn(self.inst)
	end
	
	self.rate = self.rate * self:GetRateModifier()
	
	--print (string.format("dapper: %2.2f light: %2.2f aura: %2.2f boat: %2.2f poison: %2.2f TOTAL: %2.2f", dapper_delta, light_delta, aura_delta, drivabledelta, poisondelta, self.rate*dt))
	self:DoDelta(self.rate*dt, true)
end

local function RecalcSW(self, dt)
	local total_dapperness = self.dapperness or 0
	local mitigates_rain = false
	
	local empty_slots = 3
	local tempDappernessVal = 0
	for k,v in pairs (self.inst.components.inventory.equipslots) do
		if v.components.equippable then
			empty_slots = empty_slots - 1
			-- This has been modified, to apply gain/loss per item evaluated, instead of doing it on the total
			tempDappernessVal = v.components.equippable:GetDapperness(self.inst)
			if tempDappernessVal > 0.0 then
				total_dapperness = total_dapperness + ((tempDappernessVal / 100) * dapperGainEffect)
			elseif tempDappernessVal < 0.0 then
				total_dapperness = total_dapperness + ((tempDappernessVal / 100) * dapperLossEffect)
			end
		end		
	end
	
	total_dapperness = total_dapperness * self.dapperness_mult 

	local moisture_delta = self:GetMoistureDelta()

	local dapper_delta = total_dapperness*_T.SANITY_DAPPERNESS
	
	local light_delta = 0
	local lightval = self.inst.LightWatcher:GetLightValue()
	
	local day = _G.GetClock():IsDay() and not _G.GetWorld():IsCave()
	
	if day then 
		light_delta = _T.SANITY_DAY_GAIN
	else	
		local highval = _T.SANITY_HIGH_LIGHT
		local lowval = _T.SANITY_LOW_LIGHT

		if lightval > highval then
			light_delta =  _T.SANITY_NIGHT_LIGHT
		elseif lightval < lowval then
			light_delta = _T.SANITY_NIGHT_DARK
		else
			light_delta = _T.SANITY_NIGHT_MID
		end

		light_delta = light_delta*self.night_drain_mult
	end
	
	local aura_delta = 0
	local x,y,z = self.inst.Transform:GetWorldPosition()
	local ents = _G.TheSim:FindEntities(x,y,z, _T.SANITY_EFFECT_RANGE, nil, {"FX", "NOCLICK", "DECOR","INLIMBO"} )
	for k,v in pairs(ents) do 
		if v.components.sanityaura and v ~= self.inst then
			local distsq = self.inst:GetDistanceSqToInst(v)
			local aura_val = v.components.sanityaura:GetAura(self.inst)/math.max(1, distsq)
			if aura_val < 0 then
				aura_val = aura_val * self.neg_aura_mult
			end
			
			-- This is customized, to apply gain/loss per aura evaluated, instead of doing it on the total
			if aura_val > 0.0 then
				aura_val = ((aura_val / 100) * auraGainEffect)
			elseif aura_val < 0.0 then
				aura_val = ((aura_val / 100) * auraLossEffect)
			end

			aura_delta = aura_delta + aura_val
		end
	end
	
 	local drivabledelta = 0 
 	if self.inst.components.driver then 
 		local vehicle = self.inst.components.driver.vehicle
 		if vehicle then 
 			if vehicle.components.drivable then
 				drivabledelta = vehicle.components.drivable:GetSanityDrain()
			elseif vehicle.components.searchable then
				drivabledelta = vehicle.components.searchable:GetSanityDrain()
			end
 		end 
 	end 

 	local poisondelta = 0
 	if self.inst.components.poisonable and self.inst.components.poisonable:IsPoisoned() then
 		poisondelta = -self.inst.components.poisonable.damage_per_interval * _T.POISON_SANITY_SCALE
 	end
	
	drivabledelta = ((drivabledelta / 100) * drivableEffect)
	poisondelta = ((poisondelta / 100) * poisonEffect)
	moisture_delta = ((moisture_delta / 100) * rainEffect)
 
	self.rate = (dapper_delta + moisture_delta + light_delta + aura_delta + drivabledelta + poisondelta)
	
	if self.custom_rate_fn then
		self.rate = self.rate + self.custom_rate_fn(self.inst)
	end
	
	self.rate = self.rate * self:GetRateModifier()
	
	--print (string.format("dapper: %2.2f light: %2.2f aura: %2.2f boat: %2.2f poison: %2.2f TOTAL: %2.2f", dapper_delta, light_delta, aura_delta, drivabledelta, poisondelta, self.rate*dt))
	self:DoDelta(self.rate*dt, true)
end

local function RecalcRoG(self, dt)
	local total_dapperness = self.dapperness or 0
	local mitigates_rain = false
	
	local empty_slots = 3
	local tempDappernessVal = 0
	for k,v in pairs (self.inst.components.inventory.equipslots) do
		if v.components.equippable then
			empty_slots = empty_slots - 1
			-- This has been modified, to apply gain/loss per item evaluated, instead of doing it on the total
			tempDappernessVal = v.components.equippable:GetDapperness(self.inst)
			if tempDappernessVal > 0.0 then
				total_dapperness = total_dapperness + ((tempDappernessVal / 100) * dapperGainEffect)
			elseif tempDappernessVal < 0.0 then
				total_dapperness = total_dapperness + ((tempDappernessVal / 100) * dapperLossEffect)
			end
		end		
	end
	
	total_dapperness = total_dapperness * self.dapperness_mult 

	local moisture_delta = self:GetMoistureDelta()

	local dapper_delta = total_dapperness*_T.SANITY_DAPPERNESS
	
	local light_delta = 0
	local lightval = self.inst.LightWatcher:GetLightValue()
	
	local day = _G.GetClock():IsDay() and not _G.GetWorld():IsCave()
	
	if day then 
		light_delta = _T.SANITY_DAY_GAIN
	else	
		local highval = _T.SANITY_HIGH_LIGHT
		local lowval = _T.SANITY_LOW_LIGHT

		if lightval > highval then
			light_delta =  _T.SANITY_NIGHT_LIGHT
		elseif lightval < lowval then
			light_delta = _T.SANITY_NIGHT_DARK
		else
			light_delta = _T.SANITY_NIGHT_MID
		end

		light_delta = light_delta*self.night_drain_mult
	end
	
	local aura_delta = 0
	local x,y,z = self.inst.Transform:GetWorldPosition()
	local ents = _G.TheSim:FindEntities(x,y,z, _T.SANITY_EFFECT_RANGE, nil, {"FX", "NOCLICK", "DECOR","INLIMBO"} )
	for k,v in pairs(ents) do 
		if v.components.sanityaura and v ~= self.inst then
			local distsq = self.inst:GetDistanceSqToInst(v)
			local aura_val = v.components.sanityaura:GetAura(self.inst)/math.max(1, distsq)
			if aura_val < 0 then
				aura_val = aura_val * self.neg_aura_mult
			end
			
			-- This is customized, to apply gain/loss per aura evaluated, instead of doing it on the total
			if aura_val > 0.0 then
				aura_val = ((aura_val / 100) * auraGainEffect)
			elseif aura_val < 0.0 then
				aura_val = ((aura_val / 100) * auraLossEffect)
			end

			aura_delta = aura_delta + aura_val
		end
	end
	
	moisture_delta = ((moisture_delta / 100) * rainEffect)
 
	self.rate = (dapper_delta + moisture_delta + light_delta + aura_delta )	
	
	if self.custom_rate_fn then
		self.rate = self.rate + self.custom_rate_fn(self.inst)
	end

	--print (string.format("dapper: %2.2f light: %2.2f TOTAL: %2.2f", dapper_delta, light_delta, self.rate*dt))
	self:DoDelta(self.rate*dt, true)
end

local function RecalcDS(self, dt)
	local total_dapperness = self.dapperness or 0
	local mitigates_rain = false
	local tempDappernessVal = 0
	for k,v in pairs (self.inst.components.inventory.equipslots) do
		if v.components.dapperness then
			-- This has been modified, to apply gain/loss per item evaluated, instead of doing it on the total
			tempDappernessVal = v.components.dapperness:GetDapperness(self.inst)
			if tempDappernessVal > 0.0 then
				total_dapperness = total_dapperness + ((tempDappernessVal / 100) * dapperGainEffect)
			elseif tempDappernessVal < 0.0 then
				total_dapperness = total_dapperness + ((tempDappernessVal / 100) * dapperLossEffect)
			end
			
			if v.components.dapperness.mitigates_rain then
				mitigates_rain = true
			end
		end		
	end
	
	local dapper_delta = total_dapperness*_T.SANITY_DAPPERNESS
	
	local light_delta = 0
	local lightval = self.inst.LightWatcher:GetLightValue()
	
	local day = _G.GetClock():IsDay() and not _G.GetWorld():IsCave()
	
	if day then 
		light_delta = _T.SANITY_DAY_GAIN
	else	
		local highval = _T.SANITY_HIGH_LIGHT
		local lowval = _T.SANITY_LOW_LIGHT

		if lightval > highval then
			light_delta =  _T.SANITY_NIGHT_LIGHT
		elseif lightval < lowval then
			light_delta = _T.SANITY_NIGHT_DARK
		else
			light_delta = _T.SANITY_NIGHT_MID
		end

		light_delta = light_delta*self.night_drain_mult
	end
	
	local aura_delta = 0
	local x,y,z = self.inst.Transform:GetWorldPosition()
	local ents = _G.TheSim:FindEntities(x,y,z, _T.SANITY_EFFECT_RANGE, nil, {"FX", "NOCLICK", "DECOR","INLIMBO"} )
	for k,v in pairs(ents) do 
		if v.components.sanityaura and v ~= self.inst then
			local distsq = self.inst:GetDistanceSqToInst(v)
			local aura_val = v.components.sanityaura:GetAura(self.inst)/math.max(1, distsq)
			if aura_val < 0 then
				aura_val = aura_val * self.neg_aura_mult
			end
			
			-- This is customized, to apply gain/loss per aura evaluated, instead of doing it on the total
			if aura_val > 0.0 then
				aura_val = ((aura_val / 100) * auraGainEffect)
			elseif aura_val < 0.0 then
				aura_val = ((aura_val / 100) * auraLossEffect)
			end

			aura_delta = aura_delta + aura_val
		end
	end


	local rain_delta = 0
	if _G.GetSeasonManager() and _G.GetSeasonManager():IsRaining() and not mitigates_rain then
		rain_delta = -_T.DAPPERNESS_MED*1.5* _G.GetSeasonManager():GetPrecipitationRate()
	end
	
	rain_delta = ((rain_delta / 100) * rainEffect)
	
	self.rate = (dapper_delta + light_delta + aura_delta + rain_delta)	
	
	if self.custom_rate_fn then
		self.rate = self.rate + self.custom_rate_fn(self.inst)
	end

	--print (string.format("dapper: %2.2f light: %2.2f TOTAL: %2.2f", dapper_delta, light_delta, self.rate*dt))
	self:DoDelta(self.rate*dt, true)
end

local function ChangeRecalcHam(player_inst)
	if player_inst.components.sanity then
		player_inst.components.sanity.Recalc = RecalcHam
	end
end

local function ChangeRecalcSW(player_inst)
	if player_inst.components.sanity then
		player_inst.components.sanity.Recalc = RecalcSW
	end
end

local function ChangeRecalcRoG(player_inst)
	if player_inst.components.sanity then
		player_inst.components.sanity.Recalc = RecalcRoG
	end
end

local function ChangeRecalcDS(player_inst)
	if player_inst.components.sanity then
		player_inst.components.sanity.Recalc = RecalcDS
	end
end

if IsHamlet then AddPlayerPostInit(ChangeRecalcHam)
elseif IsSW then AddPlayerPostInit(ChangeRecalcSW)
elseif IsRoG then AddPlayerPostInit(ChangeRecalcRoG)
else AddPlayerPostInit(ChangeRecalcDS) end

if flowerSanityChange ~= GLOBAL.TUNING.SANITY_TINY then
	local flowers = {"flower", "flower_rainforest"}
	for i, v in ipairs(flowers) do
		AddPrefabPostInit(v, function(inst)
			local oldonpickedfn = inst.components.pickable.onpickedfn
			inst.components.pickable.onpickedfn = function(inst, picker)
				if oldonpickedfn then
					oldonpickedfn(inst, picker)
				end
				if picker ~= nil and picker.components.sanity ~= nil then
					picker.components.sanity:DoDelta(flowerSanityChange - (picker:HasTag("plantkin") and 0 or TUNING.SANITY_TINY))
				end
			end
		end)
	end
end

AddPrefabPostInit("seashell_beached", function(inst)
	local oldonpickedfn = inst.components.pickable.onpickedfn
	inst.components.pickable.onpickedfn = function(inst, picker)
		if oldonpickedfn then
			oldonpickedfn(inst, picker)
		end
		if picker ~= nil and picker.components.sanity ~= nil then
			picker.components.sanity:DoDelta(seashellSanityChange - TUNING.SANITY_TINY)
		end
	end
end)

if evilFlowerSanityChange ~= -GLOBAL.TUNING.SANITY_TINY then
	AddPrefabPostInit("flower_evil", function(inst)
		local oldonpickedfn = inst.components.pickable.onpickedfn
		inst.components.pickable.onpickedfn = function(inst, picker)
			if oldonpickedfn then
				oldonpickedfn(inst, picker)
			end
			if picker ~= nil and picker.components.sanity ~= nil then
				picker.components.sanity:DoDelta(evilFlowerSanityChange + TUNING.SANITY_TINY)
			end
		end
	end)
end